package com.qs.mp.web.controller.api.channel;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.param.ChannelParam;
import com.qs.mp.channel.domain.param.VerifyParam;
import com.qs.mp.channel.domain.vo.ChannelOperDataVO;
import com.qs.mp.channel.domain.vo.ChannelVO;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.ChannelCertifyStatusEnum;
import com.qs.mp.common.enums.ChannelVerifyStatusEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import ma.glasnost.orika.MapperFacade;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @auther zhongcp
 * @create 2022-02-28 16:17:48
 * @describe 渠道管理前端控制器
 */
@Api("渠道管理API")
@RestController
@RequestMapping("/api/v1/mp/channel/*")
@Component
public class ChannelController extends BaseApiController {

	@Autowired
	private IChannelService channelService;

	@Autowired
	private MapperFacade mapperFacade;

	/**
	 * 获取我的下级渠道列表信息，支持翻页
	 *
	 * @return
	 */
	@PostMapping("subchannel/list")
	public TableDataInfo listChannel(@RequestBody Channel channel) {
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		List<ChannelVO> list = new ArrayList<ChannelVO>();
		if(null != channelId) {
			channel.setParentId(channelId);
			startPage();
			list = channelService.selectChannelVoList(channel,"channel");
		}
		return getDataTable(list);
	}

	/**
	 * 获取我的下级渠道详情信息
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "subchannel/detail")
	public AjaxResult getChannelDetail(@RequestBody Channel channel) {
		if (null == channel || null == channel.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Long parentlId = SecurityUtils.getLoginUser().getChannelId();
		ChannelVO channelVO = new ChannelVO();
		Channel queryChannel = channelService.getById(channel.getChannelId());
		if(null == queryChannel || null == queryChannel.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		if(!parentlId.equals(queryChannel.getParentId())) {
			return AjaxResult.error("非当前用户的子渠道,查询失败");
		}
	    BeanUtils.copyProperties(queryChannel, channelVO);
	    // 查询今日的销售额、佣金收入、新增用户数
	    ChannelOperDataVO channelOperDataVO = channelService.getChannelOperData(channelVO.getChannelNo(), 1);
	    channelVO.setOperData(channelOperDataVO);
		return AjaxResult.success(channelVO);
	}

	/**
	 * 新增子渠道信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "新增子渠道信息", notes = "渠道端新增子渠道")
	@PostMapping("subchannel/create")
	public AjaxResult channelCreate(@Validated @RequestBody ChannelParam channelParam) {
		if (channelParam.getChannelId() != null && channelParam.getChannelId() != 0) {
			return AjaxResult.error("该渠道已存在");
		}
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		Channel channel = mapperFacade.map(channelParam, Channel.class);
		channel.setParentId(channelId);
		try {
			channelService.saveChannel(channel, "channel");
		} catch (Exception e) {
			return AjaxResult.error("子渠道'" + channel.getName() + "'新增失败");
		}
		return AjaxResult.success("子渠道'" + channel.getName() + "'新增成功");
	}

	/**
	 * 编辑子渠道信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "编辑子渠道信息", notes = "渠道端编辑子渠道")
	@PostMapping("subchannel/update")
	public AjaxResult channelUpdate(@Validated @RequestBody ChannelParam channelParam) {

		if (null == channelParam || null == channelParam.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Channel channel = mapperFacade.map(channelParam, Channel.class);
		try {
			channelService.updateChannel(channel, "channel");
		} catch (Exception e) {
			return AjaxResult.error("子渠道'" + channel.getName() + "'编辑失败");
		}
		return AjaxResult.success("子渠道'" + channel.getName() + "'编辑成功");
	}
	
	
	/**
	 *获取当前用户渠道或者某个子渠道的经营数据
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "subchannel/operdata/query")
	public AjaxResult getOperData(@RequestBody JSONObject jsonObject) {
		if (null == jsonObject) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		int days = jsonObject.getIntValue("days");;
		if(StringUtils.isNotBlank(jsonObject.getString("channelId"))) {
			Long _channelId = jsonObject.getLong("channelId");
			Channel queryChannel = channelService.getById(_channelId);
			if(null == queryChannel || null == queryChannel.getChannelId()) {
				return error(ErrorCodeEnum.ERROR_CODE_1001);
			}
			if(!channelId.equals(queryChannel.getParentId())) {
				return AjaxResult.error("非当前用户的子渠道,查询失败");
			}
			channelId = _channelId;
		}
		days = days > 0?days:1;
		days = days > 0?days:1;
		Channel channel = channelService.getById(channelId);
	    // 查询今日的销售额、佣金收入、新增用户数
	    ChannelOperDataVO channelOperDataVO = channelService.getChannelOperData(channel.getChannelNo(), 1);
		return AjaxResult.success(channelOperDataVO);
	}
	
	/**
	 * 获取子渠道的经营数据列表
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "subchannel/operdata/list")
	public TableDataInfo getOperDataList(@RequestBody JSONObject jsonObject) {
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		int days = jsonObject.getIntValue("days");;
		if(StringUtils.isNotBlank(jsonObject.getString("channelId"))) {
			channelId = jsonObject.getLong("channelId");
		}
		days = days > 0?days:1;
		List<Channel> list = channelService.list(
		        new LambdaQueryWrapper<Channel>().eq(Channel::getParentId, channelId)
		            .orderByDesc(Channel::getChannelId));
		
		List<ChannelOperDataVO> dataList = new ArrayList<ChannelOperDataVO>();
		if(null != list && list.size() > 0) {
			for(Channel channel : list) {
				if(null != channel && null != channel.getChannelNo()) {
				    // 查询今日的销售额、佣金收入、新增用户数
				    ChannelOperDataVO channelOperDataVO = channelService.getChannelOperData(channel.getChannelNo(), days);
				    if(null != channelOperDataVO) {
				    	channelOperDataVO.setName(channel.getName());
				    	dataList.add(channelOperDataVO);
				    }
				}
			}
		}
		return getDataTable(dataList);
	}
	
	
	/**
	 * 获取我的经销商列表信息，支持翻页
	 *
	 * @return
	 */
	@PostMapping("site/list")
	public TableDataInfo listSite(@RequestBody Channel channel) {
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		List<ChannelVO> list = new ArrayList<ChannelVO>();
		if(null != channelId) {
			channel.setParentId(channelId);
			startPage();
			list = channelService.selectChannelVoList(channel,"channel");
		}
		return getDataTable(list);
	}

	/**
	 * 获取我的下级渠道详情信息
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "site/detail")
	public AjaxResult getSiteDetail(@RequestBody Channel channel) {
		if (null == channel || null == channel.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Long parentlId = SecurityUtils.getLoginUser().getChannelId();
		ChannelVO channelVO = new ChannelVO();
		Channel queryChannel = channelService.getById(channel.getChannelId());
		if(null == queryChannel || null == queryChannel.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		if(!parentlId.equals(queryChannel.getParentId())) {
			return AjaxResult.error("非当前用户的经销商,查询失败");
		}
	    BeanUtils.copyProperties(queryChannel, channelVO);
	    // 查询今日的销售额、佣金收入、新增用户数
	    ChannelOperDataVO channelOperDataVO = channelService.getChannelOperData(channelVO.getChannelNo(), 1);
	    channelVO.setOperData(channelOperDataVO);
		return AjaxResult.success(channelVO);
	}

	/**
	 * 新增经销商信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "渠道端新增经销商", notes = "渠道端新增经销商")
	@PostMapping("site/create")
	public AjaxResult siteCreate(@Validated @RequestBody ChannelParam channelParam) {
		if (channelParam.getChannelId() != null && channelParam.getChannelId() != 0) {
			return AjaxResult.error("该经销商已存在");
		}
		Long channelId = SecurityUtils.getLoginUser().getChannelId();

		Channel channel = mapperFacade.map(channelParam, Channel.class);
		channel.setParentId(channelId);
		try {
			channelService.saveChannel(channel,"site");
		} catch (Exception e) {
			return AjaxResult.error("经销商'" + channel.getName() + "'新增失败");
		}
		return AjaxResult.success("经销商'" + channel.getName() + "'新增成功");
	}

	/**
	 * 编辑经销商信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "编辑经销商信息", notes = "渠道端编辑经销商")
	@PostMapping("site/update")
	public AjaxResult siteUpdate(@Validated @RequestBody ChannelParam channelParam) {

		if (null == channelParam || null == channelParam.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Channel channel = mapperFacade.map(channelParam, Channel.class);
		try {
			channelService.updateChannel(channel,"site");
		} catch (Exception e) {
			return AjaxResult.error("经销商'" + channel.getName() + "'编辑失败");
		}
		return AjaxResult.success("经销商'" + channel.getName() + "'编辑成功");
	}
	
	
	/**
	 * 经销商认证审核
	 * @param
	 * @return
	 */
	@ApiOperation(value = "经销商认证审核", notes = "经销商认证审核")
	@PostMapping("site/verify")
	public AjaxResult siteVerify(@RequestBody Channel channel) {

		if (null == channel || null == channel.getChannelId() || null == channel.getVerifyStatus()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		
		Long parentlId = SecurityUtils.getLoginUser().getChannelId();
		Channel queryChannel = channelService.getById(channel.getChannelId());
		if(null == queryChannel || null == queryChannel.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		if(!parentlId.equals(queryChannel.getParentId())) {
			return AjaxResult.error(queryChannel.getName() + "非当前用户的经销商, 认证审核失败");
		}
		try {
			if(channel.getVerifyStatus() == ChannelVerifyStatusEnum.ACCEPT) {
				channel.setCertifyStatus(ChannelCertifyStatusEnum.CERTIFIED);
			}else {
				channel.setCertifyStatus(ChannelCertifyStatusEnum.NOT_CERTIFIED);
			}
			channelService.updateById(channel);
		} catch (Exception e) {
			return AjaxResult.error("经销商'" + channel.getName() + "'认证审核失败");
		}
		return AjaxResult.success("经销商'" + channel.getName() + "'认证审核成功");
	}
	
	/**
	 * 经销商认证页面点提交
	 * @param
	 * @return
	 */
	@ApiOperation(value = "经销商认证信息提交", notes = "经销商认证页面点提交")
	@PostMapping("site/verify/submit")
	public AjaxResult siteVerifySubmit(@Validated @RequestBody VerifyParam verifyParam) {

		if (null == verifyParam || null == verifyParam.getChannelId()) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Channel channel = mapperFacade.map(verifyParam, Channel.class);
		try {
			channel.setVerifyStatus(ChannelVerifyStatusEnum.WAIT);
			channelService.updateById(channel);
		} catch (Exception e) {
			return AjaxResult.error("经销商'" + channel.getName() + "'提交认证信息失败");
		}
		return AjaxResult.success("经销商'" + channel.getName() + "'提交认证信息成功");
	}
	
	
	/**
	 * 查看我的渠道信息
	 *
	 * @param
	 * @return
	 */
	@PostMapping(value = "mine/detail")
	@ApiOperation(value = "查看我的渠道信息", notes = "渠道端我的里面点击佣金比例")
	public AjaxResult getMineDetail() {
		Long channelId = SecurityUtils.getLoginUser().getChannelId();
		if (null == channelId) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		ChannelVO channelVO = new ChannelVO();
		Channel queryChannel = channelService.getById(channelId);
	    BeanUtils.copyProperties(queryChannel, channelVO);
	    // 查询今日的销售额、佣金收入、新增用户数
	    Channel parentChannel = channelService.getById(queryChannel.getParentId());
	    channelVO.setParentName(parentChannel.getName());
		return AjaxResult.success(channelVO);
	}
}
